// our headers
#include "util.h"
#include "charcontrol.h"
#include "modelviewer.h"
#include "itemselection.h"

#ifdef _WIN32
	using namespace stdext;
#endif

enum {
	SPIN_SKIN_COLOR,
	SPIN_FACE_TYPE,
	SPIN_HAIR_COLOR,
	SPIN_HAIR_STYLE,
	SPIN_FACIAL_HAIR,
	SPIN_FACIAL_COLOR,

	NUM_SPIN_BTNS
};

enum {
	SPIN_TABARD_ICON = 0,
	SPIN_TABARD_ICONCOLOR,
	SPIN_TABARD_BORDER,
	SPIN_TABARD_BORDERCOLOR,
	SPIN_TABARD_BACKGROUND,

	NUM_TABARD_BTNS
};


IMPLEMENT_CLASS(CharControl, wxWindow)


BEGIN_EVENT_TABLE(CharControl, wxWindow)
	EVT_SIZE(CharControl::OnSize)

	EVT_SPIN(ID_SKIN_COLOR, CharControl::OnSpin)
	EVT_SPIN(ID_FACE_TYPE, CharControl::OnSpin)
	EVT_SPIN(ID_HAIR_COLOR, CharControl::OnSpin)
	EVT_SPIN(ID_HAIR_STYLE, CharControl::OnSpin)
	EVT_SPIN(ID_FACIAL_HAIR, CharControl::OnSpin)
	EVT_SPIN(ID_FACIAL_COLOR, CharControl::OnSpin)

	EVT_SPIN(ID_TABARD_ICON, CharControl::OnTabardSpin)
	EVT_SPIN(ID_TABARD_ICONCOLOR, CharControl::OnTabardSpin)
	EVT_SPIN(ID_TABARD_BORDER, CharControl::OnTabardSpin)
	EVT_SPIN(ID_TABARD_BORDERCOLOR, CharControl::OnTabardSpin)
	EVT_SPIN(ID_TABARD_BACKGROUND, CharControl::OnTabardSpin)
	
	/*
	EVT_CHECKBOX(ID_SHOW_UNDERWEAR, CharControl::OnCheck)
	EVT_CHECKBOX(ID_SHOW_EARS, CharControl::OnCheck)
	EVT_CHECKBOX(ID_SHOW_HAIR, CharControl::OnCheck)

	EVT_BUTTON(ID_SAVE_EQUIPMENT, CharControl::OnButton)
	EVT_BUTTON(ID_LOAD_EQUIPMENT, CharControl::OnButton)
	EVT_BUTTON(ID_CLEAR_EQUIPMENT, CharControl::OnButton)

	EVT_BUTTON(ID_LOAD_SET, CharControl::OnButton)
	EVT_BUTTON(ID_LOAD_START, CharControl::OnButton)
	*/

	EVT_BUTTON(ID_MOUNT, CharControl::OnButton)

	EVT_BUTTON(ID_EQUIPMENT + CS_HEAD, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_NECK, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_SHOULDER, CharControl::OnButton)
	
	EVT_BUTTON(ID_EQUIPMENT + CS_SHIRT, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_CHEST, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_BELT, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_PANTS, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_BOOTS, CharControl::OnButton)

	EVT_BUTTON(ID_EQUIPMENT + CS_BRACERS, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_GLOVES, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_CAPE, CharControl::OnButton)

	EVT_BUTTON(ID_EQUIPMENT + CS_HAND_RIGHT, CharControl::OnButton)
	EVT_BUTTON(ID_EQUIPMENT + CS_HAND_LEFT, CharControl::OnButton)

	EVT_BUTTON(ID_EQUIPMENT + CS_TABARD, CharControl::OnButton)
END_EVENT_TABLE()

CharControl::CharControl(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
	//:items("items.csv")
{
	// If they are using the Test Patch and a test item list exist,  use it instead.
	if(useTestPatch && wxFile::Exists("testitems.csv"))
		items.open("testitems.csv");
	else
		items.open("items.csv");

	if(Create(parent, id, pos, size, 0, _T("CharControlFrame")) == false) {
		wxMessageBox("Failed to create a window frame for the Character Control!", "Error");
		wxLogMessage("Failed to create a window frame for the Character Control!");
		return;
	}

	wxFlexGridSizer *top = new wxFlexGridSizer(1);
	top->AddGrowableCol(0);

	wxGridSizer *gs = new wxGridSizer(2, 5, 5);

	#define ADD_CONTROLS(type, id, caption) \
	gs->Add(new wxStaticText(this, -1, caption), wxSizerFlags().Align(wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL)); \
	gs->Add(spins[type] = new wxSpinButton(this, id, wxDefaultPosition, wxDefaultSize, wxSP_HORIZONTAL|wxSP_WRAP), wxSizerFlags(1).Align(wxALIGN_CENTER_VERTICAL));

	ADD_CONTROLS(SPIN_SKIN_COLOR, ID_SKIN_COLOR, _("Skin color"))
	ADD_CONTROLS(SPIN_FACE_TYPE, ID_FACE_TYPE, _("Face type"))
	ADD_CONTROLS(SPIN_HAIR_COLOR, ID_HAIR_COLOR, _("Hair color"))
	ADD_CONTROLS(SPIN_HAIR_STYLE, ID_HAIR_STYLE, _("Hair style"))
	ADD_CONTROLS(SPIN_FACIAL_HAIR, ID_FACIAL_HAIR, _("Facial hair/deco"))
	ADD_CONTROLS(SPIN_FACIAL_COLOR, ID_FACIAL_COLOR, _("Facial deco color"))
	#undef ADD_CONTROLS
	
	top->Add(gs,wxSizerFlags().Proportion(1).Expand().Border(wxALL, 10));

	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		buttons[i] = NULL;
		labels[i] = NULL;
	}
	
	top->Add(new wxStaticText(this, -1, _T("Equipment"), wxDefaultPosition, wxSize(200,20), wxALIGN_CENTRE), wxSizerFlags().Border(wxTOP, 10));
	wxFlexGridSizer *gs2 = new wxFlexGridSizer(2, 5, 5);
	gs2->AddGrowableCol(1);

	#define ADD_CONTROLS(type, caption) \
	gs2->Add(buttons[type]=new wxButton(this, ID_EQUIPMENT + type, caption), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL)); \
	gs2->Add(labels[type]=new wxStaticText(this, -1, _("---- None ----")), wxSizerFlags().Proportion(1).Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 10));
	
	ADD_CONTROLS(CS_HEAD, _("Head"))
	//ADD_CONTROLS(CS_NECK, _T("Neck"))
	ADD_CONTROLS(CS_SHOULDER, _("Shoulder"))

	ADD_CONTROLS(CS_SHIRT, _("Shirt"))
	ADD_CONTROLS(CS_CHEST, _("Chest"))
	ADD_CONTROLS(CS_BELT, _("Belt"))
	ADD_CONTROLS(CS_PANTS, _("Legs"))
	ADD_CONTROLS(CS_BOOTS, _("Boots"))

	ADD_CONTROLS(CS_BRACERS, _("Bracers"))
	ADD_CONTROLS(CS_GLOVES, _("Gloves"))
	ADD_CONTROLS(CS_CAPE, _("Cape"))

	ADD_CONTROLS(CS_HAND_RIGHT, _("Right hand"))
	ADD_CONTROLS(CS_HAND_LEFT, _("Left hand"))

	ADD_CONTROLS(CS_TABARD, _("Tabard"))
	#undef ADD_CONTROLS
	
	top->Add(gs2, wxEXPAND);

	// Create our tabard customisation spin buttons
	wxGridSizer *gs3 = new wxGridSizer(2, 5, 5);
	#define ADD_CONTROLS(type, id, caption) \
	gs3->Add(new wxStaticText(this, -1, caption), wxSizerFlags().Align(wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL)); \
	gs3->Add(tabardSpins[type]=new wxSpinButton(this, id, wxDefaultPosition, wxDefaultSize, wxSP_HORIZONTAL|wxSP_WRAP), wxSizerFlags(1).Align(wxALIGN_CENTER_VERTICAL));

	ADD_CONTROLS(SPIN_TABARD_ICON, ID_TABARD_ICON, _("Icon"))
	ADD_CONTROLS(SPIN_TABARD_ICONCOLOR, ID_TABARD_ICONCOLOR, _("Icon Color"))
	ADD_CONTROLS(SPIN_TABARD_BORDER, ID_TABARD_BORDER, _("Border"))
	ADD_CONTROLS(SPIN_TABARD_BORDERCOLOR, ID_TABARD_BORDERCOLOR, _("Border Color"))
	ADD_CONTROLS(SPIN_TABARD_BACKGROUND, ID_TABARD_BACKGROUND, _("Background Color"))

	#undef ADD_CONTROLS

	top->Add(new wxStaticText(this, -1, _("Tabard details")), wxSizerFlags().Align(wxALIGN_CENTRE).Border(wxALL, 1));
	top->Add(gs3, wxEXPAND);
	top->Add(new wxButton(this, ID_MOUNT, _("Choose mount")), wxSizerFlags().Align(wxALIGN_CENTRE).Border(wxTOP, 15));

	//p->SetSizer(top);
	
	top->SetSizeHints(this);
	Show(true);
	SetAutoLayout(true);
	SetSizer(top);
	Layout();

	randomLooks = true;
	
	choosingSlot = 0;
	itemDialog = 0;
}

CharControl::~CharControl()
{
	// unload whatever we need to
}

void CharControl::OnSize(wxSizeEvent &event)
{
	//p->SetSize(this->GetSize());
}


bool CharControl::Init()
{
	chartex = 0;
	hairtex = 0;
	furtex = 0;
	capetex = 0;
	gobtex = 0;

	td.showCustom = false;
	bSheathe = false;

	cd.useNPC = 0;
	cd.showEars = true;
	cd.showHair = true;
	cd.showFacialHair = true;
	cd.showUnderwear = true;

	// set max values for custom tabard
	td.maxBackground = 50;
	td.maxBorder = 9;
	td.maxBorderColor = 16;
	td.maxIcon = 169;
	td.maxIconColor = 16;

	if(!hairdb.open()) return false;
	if(!chardb.open()) return false;
	if(!racedb.open()) return false;
	if(!classdb.open()) return false;
	if(!facialhairdb.open()) return false;
	if(!itemdb.open()) return false;
	if(!visualdb.open()) return false;
	if(!effectdb.open()) return false;
	if(!subclassdb.open()) return false;
	if(!sets.open()) return false;
	if(!start.open()) return false;
	//if(!helmetdb.open()) return false;
	if(!npcdb.open()) return false;

	items.cleanup(itemdb);
	sets.cleanup(items);

	return true;
}

//void CharControl::UpdateModel(Model *m)
void CharControl::UpdateModel(Attachment *a)
{
	charAtt = a;
	model = (Model*)a->model;

	if (chartex==0) 
		glGenTextures(1, &chartex);

	cd.reset();
	td.showCustom = false;

	// hide most geosets
	for (size_t i=0; i<model->geosets.size(); i++) {
		int id = model->geosets[i].id;
		model->showGeosets[i] = (id==0);
	}

	size_t p1 = model->name.find_first_of('\\', 0);
	size_t p2 = model->name.find_first_of('\\', p1+1);
	size_t p3 = model->name.find_first_of('\\', p2+1);

	std::string raceName = model->name.substr(p1+1,p2-p1-1);
	std::string genderName = model->name.substr(p2+1,p3-p2-1);

	int race, gender;

	try {
		CharRacesDB::Record raceRec = racedb.getByName(raceName.c_str());
		race = raceRec.getUInt(CharRacesDB::RaceID);
		gender = (genderName == "Female") ? 1 : 0;
	} catch (CharRacesDB::NotFound) {
		// wtf
		race = 0;
		gender = 0;
	}

	// get max values
	cd.maxSkinColor = chardb.getColorsFor(race, gender, CharSectionsDB::SkinType, 0, cd.useNPC);
	if (cd.maxSkinColor == 0) {
		wxMessageBox("The selected character does not have any NPC skins!\nSwitching back to normal character skins.");
		cd.useNPC = 0;
		cd.maxSkinColor = chardb.getColorsFor(race, gender, CharSectionsDB::SkinType, 0, cd.useNPC);
	}
	cd.maxFaceType  = chardb.getSectionsFor(race, gender, CharSectionsDB::FaceType, 0, cd.useNPC);
	cd.maxHairColor = chardb.getColorsFor(race, gender, CharSectionsDB::HairType, 0, 0);
	cd.maxFacialHair = facialhairdb.getStylesFor(race, gender);
	cd.maxFacialColor = cd.maxHairColor;

	if (cd.useNPC)
		modelViewer->optMenu->Check(ID_USENPCSKINS, 1);
	else
		modelViewer->optMenu->Check(ID_USENPCSKINS, 0);

	cd.race = race;
	cd.gender = gender;

	std::set<int> styles;
	for (CharHairGeosetsDB::Iterator it = hairdb.begin(); it != hairdb.end(); ++it) {
		if (it->getUInt(CharHairGeosetsDB::Race)==race && it->getUInt(CharHairGeosetsDB::Gender)==gender) {
			styles.insert(it->getUInt(CharHairGeosetsDB::Section));
		}
	}
	cd.maxHairStyle = (int)styles.size();

	if (cd.maxFaceType==0) cd.maxFaceType = 1;
	if (cd.maxSkinColor==0) cd.maxSkinColor = 1;
	if (cd.maxHairColor==0) cd.maxHairColor = 1;
	if (cd.maxHairStyle==0) cd.maxHairStyle = 1;
	if (cd.maxFacialHair==0) cd.maxFacialHair = 1;
	spins[0]->SetRange(0, cd.maxSkinColor-1);
	spins[1]->SetRange(0, cd.maxFaceType-1);
	spins[2]->SetRange(0, cd.maxHairColor-1);
	spins[3]->SetRange(0, cd.maxHairStyle-1);
	spins[4]->SetRange(0, cd.maxFacialHair-1);
	spins[5]->SetRange(0, cd.maxHairColor-1);

	if (randomLooks) {
		// Choose random values for the looks! ^_^
		cd.skinColor = randint(0, cd.maxSkinColor-1);
		cd.faceType = randint(0, cd.maxFaceType-1);
		cd.hairColor = randint(0, cd.maxHairColor-1);
		cd.hairStyle = randint(0, cd.maxHairStyle-1);
		cd.facialHair = randint(0, cd.maxFacialHair-1);
		cd.facialColor = cd.hairColor;
	}

	td.Icon = randint(0, td.maxIcon);
	td.IconColor = randint(0, td.maxIconColor);
	td.Border = randint(0, td.maxBorder);
	td.BorderColor = 0;
	td.Background = randint(0, td.maxBackground);

	spins[SPIN_SKIN_COLOR]->SetValue(cd.skinColor);
	spins[SPIN_FACE_TYPE]->SetValue(cd.faceType);
	spins[SPIN_HAIR_COLOR]->SetValue(cd.hairColor);
	spins[SPIN_HAIR_STYLE]->SetValue(cd.hairStyle);
	spins[SPIN_FACIAL_HAIR]->SetValue(cd.facialHair);
	spins[SPIN_FACIAL_COLOR]->SetValue(cd.facialColor);

	tabardSpins[SPIN_TABARD_ICON]->SetValue(td.Icon);
	tabardSpins[SPIN_TABARD_ICONCOLOR]->SetValue(td.IconColor);
	tabardSpins[SPIN_TABARD_BORDER]->SetValue(td.Border);
	tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetValue(td.BorderColor);
	tabardSpins[SPIN_TABARD_BACKGROUND]->SetValue(td.Background);

	tabardSpins[SPIN_TABARD_ICON]->SetRange(0, td.maxIcon);
	tabardSpins[SPIN_TABARD_ICONCOLOR]->SetRange(0, td.maxIconColor);
	tabardSpins[SPIN_TABARD_BORDER]->SetRange(0, td.maxBorder);
	tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetRange(0, td.maxBorderColor);
	tabardSpins[SPIN_TABARD_BACKGROUND]->SetRange(0, td.maxBackground);

	for (int i=0; i<NUM_SPIN_BTNS; i++) spins[i]->Refresh(false);
	for (int i=0; i<NUM_TABARD_BTNS; i++) tabardSpins[i]->Refresh(false);

	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		if (labels[i]) labels[i]->SetLabel(_T("---- None ----"));
	}

	// HACK: for goblin males, explicitly load a hair texture
	if (race==9 && gender==0) {
        gobtex = texturemanager.add("Creature\\Goblin\\Goblin.blp");		
	} else {
		if (gobtex) texturemanager.del(gobtex);
		gobtex = 0;
	}

	RefreshModel();
}

void CharControl::UpdateNPCModel(Attachment *a, unsigned int id)
{
	charAtt = a;
	Model *m = (Model*)a->model;
	model = m;

	if (chartex==0) 
		glGenTextures(1, &chartex);

	cd.reset();
	td.showCustom = false;

	// hide most geosets
	for (size_t i=0; i<model->geosets.size(); i++) {
		int id = model->geosets[i].id;
		model->showGeosets[i] = (id==0);
	}

	size_t p1 = m->name.find_first_of('\\', 0);
	size_t p2 = m->name.find_first_of('\\', p1+1);
	size_t p3 = m->name.find_first_of('\\', p2+1);

	std::string raceName = m->name.substr(p1+1,p2-p1-1);
	std::string genderName = m->name.substr(p2+1,p3-p2-1);

	int race, gender;

	try {
		CharRacesDB::Record raceRec = racedb.getByName(raceName.c_str());
		race = raceRec.getUInt(CharRacesDB::RaceID);
		gender = (genderName == "Female") ? 1 : 0;
	} catch (CharRacesDB::NotFound) {
		// wtf
		race = 0;
		gender = 0;
	}

	cd.race = race;
	cd.gender = gender;

	NPCDB::Record npcrec = npcdb.getByNPCID(id);

	// facial hair geosets
	try {
		cd.hairColor = npcrec.getUInt(NPCDB::HairColor);
		cd.hairStyle = npcrec.getUInt(NPCDB::HairStyle);
		cd.facialHair = npcrec.getUInt(NPCDB::FacialHair);
		cd.facialColor = cd.hairColor;
	} catch (NPCDB::NotFound) {}

	// Equip our npc
	try {
		cd.equipment[CS_HEAD] = items.getItemNum(npcrec.getUInt(NPCDB::HelmID));
		cd.equipment[CS_SHOULDER] = items.getItemNum(npcrec.getUInt(NPCDB::ShoulderID));
		cd.equipment[CS_SHIRT] = items.getItemNum(npcrec.getUInt(NPCDB::ShirtID));
		cd.equipment[CS_CHEST] = items.getItemNum(npcrec.getUInt(NPCDB::ChestID));
		cd.equipment[CS_BELT] = items.getItemNum(npcrec.getUInt(NPCDB::BeltID));
		cd.equipment[CS_PANTS] = items.getItemNum(npcrec.getUInt(NPCDB::PantsID));
		cd.equipment[CS_BOOTS] = items.getItemNum(npcrec.getUInt(NPCDB::BootsID));
		cd.equipment[CS_BRACERS] = items.getItemNum(npcrec.getUInt(NPCDB::BracersID));
		cd.equipment[CS_GLOVES] = items.getItemNum(npcrec.getUInt(NPCDB::GlovesID));
		cd.equipment[CS_TABARD] = items.getItemNum(npcrec.getUInt(NPCDB::TabardID));
		if (cd.equipment[CS_TABARD] != 0) 
			cd.geosets[12] = 2;
	} catch (...) {}

	if (cd.maxFaceType==0) cd.maxFaceType = 1;
	if (cd.maxSkinColor==0) cd.maxSkinColor = 1;
	if (cd.maxHairColor==0) cd.maxHairColor = 1;
	if (cd.maxHairStyle==0) cd.maxHairStyle = 1;
	if (cd.maxFacialHair==0) cd.maxFacialHair = 1;
	spins[0]->SetRange(0, cd.maxSkinColor-1);
	spins[1]->SetRange(0, cd.maxFaceType-1);
	spins[2]->SetRange(0, cd.maxHairColor-1);
	spins[3]->SetRange(0, cd.maxHairStyle-1);
	spins[4]->SetRange(0, cd.maxFacialHair-1);
	spins[5]->SetRange(0, cd.maxHairColor-1);


	spins[SPIN_SKIN_COLOR]->SetValue(cd.skinColor);
	spins[SPIN_FACE_TYPE]->SetValue(cd.faceType);
	spins[SPIN_HAIR_COLOR]->SetValue(cd.hairColor);
	spins[SPIN_HAIR_STYLE]->SetValue(cd.hairStyle);
	spins[SPIN_FACIAL_HAIR]->SetValue(cd.facialHair);
	spins[SPIN_FACIAL_COLOR]->SetValue(cd.facialColor);

	tabardSpins[SPIN_TABARD_ICON]->SetValue(td.Icon);
	tabardSpins[SPIN_TABARD_ICONCOLOR]->SetValue(td.IconColor);
	tabardSpins[SPIN_TABARD_BORDER]->SetValue(td.Border);
	tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetValue(td.BorderColor);
	tabardSpins[SPIN_TABARD_BACKGROUND]->SetValue(td.Background);

	tabardSpins[SPIN_TABARD_ICON]->SetRange(0, td.maxIcon);
	tabardSpins[SPIN_TABARD_ICONCOLOR]->SetRange(0, td.maxIconColor);
	tabardSpins[SPIN_TABARD_BORDER]->SetRange(0, td.maxBorder);
	tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetRange(0, td.maxBorderColor);
	tabardSpins[SPIN_TABARD_BACKGROUND]->SetRange(0, td.maxBackground);

	for (int i=0; i<NUM_SPIN_BTNS; i++) spins[i]->Refresh(false);
	for (int i=0; i<NUM_TABARD_BTNS; i++) tabardSpins[i]->Refresh(false);

	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		if (labels[i]) labels[i]->SetLabel(_T("---- None ----"));
	}

	if (gobtex)
		texturemanager.del(gobtex);
	gobtex = 0;
}

void CharControl::OnSpin(wxSpinEvent &event)
{
	if (canvas->modelType == MT_NPC)
		return;

	if (event.GetId()==ID_SKIN_COLOR) cd.skinColor = event.GetPosition();
	if (event.GetId()==ID_FACE_TYPE) cd.faceType = event.GetPosition();
	if (event.GetId()==ID_HAIR_COLOR) {
		cd.hairColor = event.GetPosition();
		cd.facialColor = event.GetPosition();
	}
	if (event.GetId()==ID_HAIR_STYLE) cd.hairStyle = event.GetPosition();
	if (event.GetId()==ID_FACIAL_HAIR) cd.facialHair = event.GetPosition();
	if (event.GetId()==ID_FACIAL_COLOR) cd.facialColor = event.GetPosition();

	RefreshModel();
}

void CharControl::OnCheck(wxCommandEvent &event)
{
	if (event.GetId()==ID_SHOW_UNDERWEAR) cd.showUnderwear = event.IsChecked();
	if (event.GetId()==ID_SHOW_HAIR) cd.showHair = event.IsChecked();
	if (event.GetId()==ID_SHOW_FACIALHAIR) cd.showFacialHair = event.IsChecked();
	if (event.GetId()==ID_SHOW_EARS) cd.showEars = event.IsChecked();
	if (event.GetId()==ID_SHEATHE) bSheathe = event.IsChecked();
	if (event.GetId()==ID_USENPCSKINS) cd.useNPC = event.IsChecked();

	RefreshEquipment();
	RefreshModel();
}

bool slotHasModel(int i)
{
	return (i==CS_HEAD || i==CS_SHOULDER || i==CS_HAND_LEFT || i==CS_HAND_RIGHT);
}

void CharControl::RefreshEquipment()
{
	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		if (slotHasModel(i)) 
			RefreshItem(i);
		if (labels[i])
			labels[i]->SetLabel(wxString(items.get(cd.equipment[i]).name.c_str(), *wxConvCurrent));
	}
}

void CharControl::OnButton(wxCommandEvent &event)
{
	bool needRefresh = true;

	if (event.GetId()==ID_SAVE_EQUIPMENT) {
		needRefresh = false;
		wxFileDialog dialog(this, _("Save equipment"), wxEmptyString, wxEmptyString, _T("Equipment files (*.eq)|*.eq"), wxSAVE|wxOVERWRITE_PROMPT);
		dialog.SetDirectory(".");
		if (dialog.ShowModal()==wxID_OK) {
			wxString s(dialog.GetPath());
			cd.save(s, &td);
		}
	} 
	else if (event.GetId()==ID_LOAD_EQUIPMENT) {
		wxFileDialog dialog(this, _("Load equipment"), wxEmptyString, wxEmptyString, _T("Equipment files (*.eq)|*.eq"), wxOPEN|wxFILE_MUST_EXIST);
		dialog.SetDirectory(".");
		if (dialog.ShowModal()==wxID_OK) {
			for (int i=0; i<NUM_CHAR_SLOTS; i++) cd.equipment[i] = 0;
			wxString s(dialog.GetPath());
			if (cd.load(s, &td)) {
				spins[SPIN_SKIN_COLOR]->SetValue(cd.skinColor);
				spins[SPIN_FACE_TYPE]->SetValue(cd.faceType);
				spins[SPIN_HAIR_COLOR]->SetValue(cd.hairColor);
				spins[SPIN_HAIR_STYLE]->SetValue(cd.hairStyle);
				spins[SPIN_FACIAL_HAIR]->SetValue(cd.facialHair);
				spins[SPIN_FACIAL_COLOR]->SetValue(cd.hairColor);
				for (int i=0; i<NUM_SPIN_BTNS; i++) spins[i]->Refresh(false);
			}
			RefreshEquipment();
		}
	}
	else if (event.GetId()==ID_CLEAR_EQUIPMENT) {
		for (int i=0; i<NUM_CHAR_SLOTS; i++) cd.equipment[i] = 0;
		RefreshEquipment();
	}
	else if (event.GetId()==ID_LOAD_SET) {
		selectSet();
	}
	else if (event.GetId()==ID_LOAD_START) {
		selectStart();
	}
	else if (event.GetId()==ID_MOUNT) {
		selectMount();
	}
	else {
		needRefresh = false;
		for (int i=0; i<NUM_CHAR_SLOTS; i++) {
			if (buttons[i] && (wxButton*)event.GetEventObject()==buttons[i]) {
				selectItem(i, cd.equipment[i], buttons[i]->GetLabel().GetData());
				break;
			}
		}
	}

	if (canvas->modelType == MT_NPC)
		RefreshNPCModel();
	else
		RefreshModel();
}


const char* regionPaths[NUM_REGIONS] =
{
	"",
	"Item\\TextureComponents\\ArmUpperTexture\\",
	"Item\\TextureComponents\\ArmLowerTexture\\",
	"Item\\TextureComponents\\HandTexture\\",
	"",
	"",
	"Item\\TextureComponents\\TorsoUpperTexture\\",
	"Item\\TextureComponents\\TorsoLowerTexture\\",
	"Item\\TextureComponents\\LegUpperTexture\\",
	"Item\\TextureComponents\\LegLowerTexture\\",
	"Item\\TextureComponents\\FootTexture\\"
};

int slotOrder[] = {	
	CS_SHIRT,
	CS_HEAD,
	CS_NECK,
	CS_SHOULDER,
	CS_PANTS,
	CS_BOOTS,
	CS_CHEST,
	CS_TABARD,
	CS_BELT,
	CS_BRACERS,
	CS_GLOVES,
	CS_HAND_RIGHT,
	CS_HAND_LEFT,
	CS_CAPE
};

int slotOrderWithRobe[] = {
	CS_SHIRT,
	CS_HEAD,
	CS_NECK,
	CS_SHOULDER,
	CS_BOOTS,
	CS_PANTS,
	CS_CHEST,
	CS_TABARD,
	CS_BELT,
	CS_BRACERS,
	CS_GLOVES,
	CS_HAND_RIGHT,
	CS_HAND_LEFT,
	CS_CAPE
};

void CharControl::RefreshModel()
{
	//cd.maxSkinColor = chardb.getColorsFor(race, gender, CharSectionsDB::SkinType, 0, cd.useNPC);
	//cd.maxFaceType  = chardb.getSectionsFor(race, gender, CharSectionsDB::FaceType, 0, cd.useNPC);
	//spins[0]->SetRange(0, cd.maxSkinColor-1);
	//spins[1]->SetRange(0, cd.maxFaceType-1);

	if (hairtex != 0) {
		texturemanager.del(hairtex);
		hairtex = 0;
	}
	if (furtex != 0) {
		texturemanager.del(furtex);
		furtex = 0;
	}
	if (capetex != 0) {
		texturemanager.del(capetex);
		capetex = 0;
	}

	// facial hair decorations
	for (int i=0; i<16; i++) cd.geosets[i] = 1;
		cd.geosets[1] = cd.geosets[2] = cd.geosets[3] = 0;

	// ears
	if (cd.showEars) cd.geosets[7] = 2;

	CharTexture tex;

	// base layer
	CharSectionsDB::Record rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::SkinType, 0, cd.skinColor, cd.useNPC);
	tex.addLayer(rec.getString(CharSectionsDB::Tex1), CR_BASE, 0);

	// Tauren fur
	const char *furtexname = rec.getString(CharSectionsDB::Tex2);
	if (strlen(furtexname)) furtex = texturemanager.add(furtexname);

	if (cd.showUnderwear) {
		try {
			rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::UnderwearType, 0, cd.skinColor, cd.useNPC);
			tex.addLayer(rec.getString(CharSectionsDB::Tex1), CR_PELVIS_UPPER, 1); // panties
			tex.addLayer(rec.getString(CharSectionsDB::Tex2), CR_TORSO_UPPER, 1); // bra
		} catch (CharSectionsDB::NotFound) {}
	}

	// face
	try {
		rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::FaceType, cd.faceType, cd.skinColor, cd.useNPC);
		tex.addLayer(rec.getString(CharSectionsDB::Tex1), CR_FACE_LOWER, 1);
		tex.addLayer(rec.getString(CharSectionsDB::Tex2), CR_FACE_UPPER, 1);
	} catch (CharSectionsDB::NotFound) {}

	// facial hair
	try {
		rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::FacialHairType, cd.facialHair, cd.facialColor, 0);
		tex.addLayer(rec.getString(CharSectionsDB::Tex1), CR_FACE_LOWER, 2);
		tex.addLayer(rec.getString(CharSectionsDB::Tex2), CR_FACE_UPPER, 2);
	} catch (CharSectionsDB::NotFound) {}

	// facial hair geosets
	try {
		CharFacialHairDB::Record frec = facialhairdb.getByParams(cd.race, cd.gender, cd.facialHair);
		cd.geosets[1] = frec.getUInt(CharFacialHairDB::Geoset100);
		cd.geosets[2] = frec.getUInt(CharFacialHairDB::Geoset200);
		cd.geosets[3] = frec.getUInt(CharFacialHairDB::Geoset300);
	} catch (CharFacialHairDB::NotFound) {}
	
	// hair + hair on face
	try {
		rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::HairType, cd.hairStyle, cd.hairColor, 0);
		const char* hairtexfn = rec.getString(CharSectionsDB::Tex1);
		if (strlen(hairtexfn)) 
			hairtex = texturemanager.add(hairtexfn);
		else {
			// oops, looks like we're missing a hair texture. Let's try with hair style #1.
			// (only a problem for orcs with no hair but some beard
			try {
				rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::HairType, 1, cd.hairColor, cd.useNPC);
				hairtexfn = rec.getString(CharSectionsDB::Tex1);
				if (strlen(hairtexfn)) 
					hairtex = texturemanager.add(hairtexfn);
				else 
					hairtex = 0;
			} catch (CharSectionsDB::NotFound) {
				// oh well, give up.
				hairtex = 0; // or chartex?
			}
		}
		tex.addLayer(rec.getString(CharSectionsDB::Tex2), CR_FACE_LOWER, 3);
		tex.addLayer(rec.getString(CharSectionsDB::Tex3), CR_FACE_UPPER, 3);
	} catch (CharSectionsDB::NotFound) {
		hairtex = 0;
	}

	bool bald = false;
	bool showHair = cd.showHair;
	bool showFacialHair = cd.showFacialHair;
	/*
	// Temporary work-around - need to do more research.
	// Check to see if we are wearing a helmet - if so, we need to hide our hair
	if (cd.equipment[CS_HEAD] != 0) {
		try {
			const ItemRecord &item = items.get(cd.equipment[CS_HEAD]);
			int type = item.type;
			if (type==IT_HEAD) {
				ItemDisplayDB::Record r = itemdb.getById(item.model);
				
				int geoID;
				if(cd.gender == 0)
					geoID = r.getUInt(ItemDisplayDB::GeosetF);
				else
					geoID = r.getUInt(ItemDisplayDB::GeosetG);

				HelmGeosetDB::Record rec = helmetdb.getById(geoID);
				char c1 = rec.getByte(HelmGeosetDB::Field1);
				char c2 = rec.getByte(HelmGeosetDB::Field2);
				char c3 = rec.getByte(HelmGeosetDB::Field3);
				unsigned char c4 = rec.getByte(HelmGeosetDB::Field4);
				unsigned char c5 = rec.getByte(HelmGeosetDB::Field5);
				
				//TODO: Work out what exactly these geosets mean and act accordingly.
				// These values point to records in HelmetGeosetVisData.dbc
				// Still not sure if the 2 columns are for male / female or
				// for facial hair / normal hair
				std::cout << "----------\n" << r.getUInt(ItemDisplayDB::GeosetF) << "\t" << r.getUInt(ItemDisplayDB::GeosetG) << "\n";
				
				std::cout << (unsigned int)rec.getByte(HelmGeosetDB::Field1) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field2) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field3) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field4) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field5) << "\n";
				rec = helmetdb.getById(r.getUInt(ItemDisplayDB::GeosetG));
				std::cout << (unsigned int)rec.getByte(HelmGeosetDB::Field1) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field2) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field3) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field4) << "\t" << (unsigned int)rec.getByte(HelmGeosetDB::Field5) << "\n";
				
				
				//std::cout << (int)c1 << " " << (int)c2 << " " << (int)c3 << " " << (unsigned int)c4 << " " << (unsigned int)c5 << "\n";

				if(c1 == 1)
					showHair = false;
					
				if(c5 == 1)
					showFacialHair = false;

				//if(r.getUInt(ItemDisplayDB::GeosetG) > 265)
				//	showFacialHair = false;
			}
		} catch (...) {}
	}
	*/

	// facial hair geosets
	try {
		CharFacialHairDB::Record frec = facialhairdb.getByParams(cd.race, cd.gender, cd.facialHair);
		cd.geosets[1] = frec.getUInt(CharFacialHairDB::Geoset100);
		cd.geosets[2] = frec.getUInt(CharFacialHairDB::Geoset200);
		cd.geosets[3] = frec.getUInt(CharFacialHairDB::Geoset300);
		
	} catch (CharFacialHairDB::NotFound) {}

	if (showFacialHair == false) {
		/*
		cd.geosets[1] = 1;
		cd.geosets[2] = 1;
		cd.geosets[3] = 1;
		*/
		
		CharRacesDB::Record race = racedb.getById(cd.race);
		string tmp = race.getString(CharRacesDB::GeoType1);
		if (tmp == "NORMAL") {
			cd.geosets[1] = 1;
			cd.geosets[2] = 1;
			cd.geosets[3] = 1;
		}
    }

	// select hairstyle geoset(s)
	for (CharHairGeosetsDB::Iterator it = hairdb.begin(); it != hairdb.end(); ++it) {
		if (it->getUInt(CharHairGeosetsDB::Race)==cd.race && it->getUInt(CharHairGeosetsDB::Gender)==cd.gender) {
			unsigned int id = it->getUInt(CharHairGeosetsDB::Geoset);
			unsigned int section = it->getUInt(CharHairGeosetsDB::Section);
			if (id!=0) {
				for (size_t j=0; j<model->geosets.size(); j++) {
					if (model->geosets[j].id == id) {
						//std::cout << "Hair:\t" << id << "\t" << section << "\t" << ((cd.hairStyle==section) && cd.showHair) << "\n";
						model->showGeosets[j] = (cd.hairStyle==section) && showHair;
					}
				}
			} else if (cd.hairStyle==section) 
				bald = true;
		}
	}
	if (!showHair)
		bald = true;
	

	// check if we have a robe on
	bool hadRobe = false;
	if (cd.equipment[CS_CHEST] != 0) {
		try {
			const ItemRecord &item = items.get(cd.equipment[CS_CHEST]);
			int type = item.type;
			if (type==IT_ROBE) {
				ItemDisplayDB::Record r = itemdb.getById(item.model);
				if (r.getUInt(ItemDisplayDB::GeosetC)==1) hadRobe = true;
			}
		} catch (...) {}
	}

	// check if we have a kilt on, just like our robes
	if (cd.equipment[CS_PANTS] != 0) {
		try {
			const ItemRecord &item = items.get(cd.equipment[CS_PANTS]);
			int type = item.type;
			if (type==IT_PANTS) {
				ItemDisplayDB::Record r = itemdb.getById(item.model);
				if (r.getUInt(ItemDisplayDB::GeosetC)==1) 
					hadRobe = true;
			}
		} catch (...) {}
	}

	// dressup
	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		int sn = hadRobe ? slotOrderWithRobe[i] : slotOrder[i];
		if (cd.equipment[sn] != 0) AddEquipment(sn, cd.equipment[sn], 10+i, tex);
	}

	// reset geosets
	for (size_t j=0; j<model->geosets.size(); j++) {
		int id = model->geosets[j].id;

		// hide top-of-head if we have hair.
		if (id == 1) model->showGeosets[j] = bald;

		for (int i=1; i<16; i++) {
			int a = i*100, b = (i+1) * 100;
			if (id>a && id<b) model->showGeosets[j] = (id == (a + cd.geosets[i]));
		}
	}

	// finalize texture
	tex.compose(chartex);
	
	// set replacable textures
	model->replaceTextures[1] = chartex;
	model->replaceTextures[2] = capetex;
	model->replaceTextures[6] = hairtex;
	model->replaceTextures[8] = furtex;
	
	model->replaceTextures[11] = gobtex;

	/*
	for (int i=0; i<40; i++) {
		model->atts[i].dr = (model->atts[i].id==cd.hairStyle);
	}
	*/
}

void CharControl::RefreshNPCModel()
{
	if (hairtex != 0) {
		texturemanager.del(hairtex);
		hairtex = 0;
	}
	if (furtex != 0) {
		texturemanager.del(furtex);
		furtex = 0;
	}
	if (capetex != 0) {
		texturemanager.del(capetex);
		capetex = 0;
	}

	for (int i=0; i<16; i++) cd.geosets[i] = 1;

	// facial hair/decorations
	cd.geosets[1] = cd.geosets[2] = cd.geosets[3] = 0;

	// show ears
	cd.geosets[7] = 2;

	CharTexture tex;

	// base layer
	tex.addLayer(customSkin, CR_BASE, 0);
	
	CharSectionsDB::Record rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::SkinType, 0, cd.skinColor, cd.useNPC);
	
	// Tauren fur
	const char *furtexname = rec.getString(CharSectionsDB::Tex2);
	if (strlen(furtexname)) furtex = texturemanager.add(furtexname);

	// facial hair geosets
	try {
		CharFacialHairDB::Record frec = facialhairdb.getByParams(cd.race, cd.gender, cd.facialHair);
		cd.geosets[1] = frec.getUInt(CharFacialHairDB::Geoset100);
		cd.geosets[2] = frec.getUInt(CharFacialHairDB::Geoset200);
		cd.geosets[3] = frec.getUInt(CharFacialHairDB::Geoset300);
	} catch (CharFacialHairDB::NotFound) {}
	
	// hair + hair on face
	try {
		CharSectionsDB::Record rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::HairType, cd.hairStyle, cd.hairColor, cd.useNPC);
		const char* hairtexfn = rec.getString(CharSectionsDB::Tex1);
		if (strlen(hairtexfn)) 
			hairtex = texturemanager.add(hairtexfn);
		else {
			try {
				rec = chardb.getByParams(cd.race, cd.gender, CharSectionsDB::HairType, 1, cd.hairColor, cd.useNPC);
				hairtexfn = rec.getString(CharSectionsDB::Tex1);
				if (strlen(hairtexfn)) 
					hairtex = texturemanager.add(hairtexfn);
				else 
					hairtex = 0;
			} catch (CharSectionsDB::NotFound) {
				hairtex = 0;
			}
		}
		tex.addLayer(rec.getString(CharSectionsDB::Tex2), CR_FACE_LOWER, 3);
		tex.addLayer(rec.getString(CharSectionsDB::Tex3), CR_FACE_UPPER, 3);
	} catch (CharSectionsDB::NotFound) {
		hairtex = 0;
	}

	bool bald = false;
	
	// select hairstyle geoset(s)
	for (CharHairGeosetsDB::Iterator it = hairdb.begin(); it != hairdb.end(); ++it) {
		if (it->getUInt(CharHairGeosetsDB::Race)==cd.race && it->getUInt(CharHairGeosetsDB::Gender)==cd.gender) {
			unsigned int id = it->getUInt(CharHairGeosetsDB::Geoset);
			unsigned int section = it->getUInt(CharHairGeosetsDB::Section);
			if (id!=0) {
				for (size_t j=0; j<model->geosets.size(); j++) {
					if (model->geosets[j].id == id) {
						//std::cout << "Hair:\t" << id << "\t" << section << "\t" << ((cd.hairStyle==section) && cd.showHair) << "\n";
						model->showGeosets[j] = (cd.hairStyle==section);
					}
				}
			} else if (cd.hairStyle == section) 
				bald = true;
		}
	}
	
	// check if we have a robe on
	bool hadRobe = false;
	if (cd.equipment[CS_CHEST] != 0) {
		try {
			const ItemRecord &item = items.get(cd.equipment[CS_CHEST]);
			int type = item.type;
			if (type==IT_ROBE) {
				ItemDisplayDB::Record r = itemdb.getById(item.model);
				if (r.getUInt(ItemDisplayDB::GeosetC)==1) hadRobe = true;
			}
		} catch (...) {}
	}
	

	/*
	// check if we have a kilt on, just like our robes
	if (cd.equipment[CS_PANTS] != 0) {
		try {
			const ItemRecord &item = items.get(cd.equipment[CS_PANTS]);
			int type = item.type;
			if (type==IT_PANTS) {
				ItemDisplayDB::Record r = itemdb.getById(item.model);
				if (r.getUInt(ItemDisplayDB::GeosetC)==1) 
					hadRobe = true;
			}
		} catch (...) {}
	}
	*/

	// dressup
	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		int sn = hadRobe ? slotOrderWithRobe[i] : slotOrder[i];
		if (cd.equipment[sn] != 0) AddEquipment(sn, cd.equipment[sn], 10+i, tex);
	}
	

	// reset geosets
	for (size_t j=0; j<model->geosets.size(); j++) {
		int id = model->geosets[j].id;
		
		// hide top-of-head if we have hair.
		if (id == 1) model->showGeosets[j] = bald;

		for (int i=1; i<16; i++) {
			int a = i*100, b = (i+1) * 100;
			if (id>a && id<b) model->showGeosets[j] = (id == (a + cd.geosets[i]));
		}
	}

	// finalize texture
	tex.compose(chartex);
	
	// set replacable textures
	model->replaceTextures[1] = chartex;
	model->replaceTextures[2] = capetex;
	model->replaceTextures[6] = hairtex;
	model->replaceTextures[8] = furtex;
	
	model->replaceTextures[11] = gobtex;
}

void CharControl::AddEquipment(int slot, int itemnum, int layer, CharTexture &tex)
{
	if (slot==CS_PANTS && cd.geosets[13]==2) return; // if we are wearing a robe, no pants for us! ^_^

	try {
		const ItemRecord &item = items.get(itemnum);
		int type = item.type;
		ItemDisplayDB::Record r = itemdb.getById(item.model);

		// shirt sleeves
		if (type==IT_SHIRT || type==IT_CHEST || type==IT_ROBE) {
			cd.geosets[8] = 1 + r.getUInt(ItemDisplayDB::GeosetA);
		}

		// pants sleeves
		if (type==IT_PANTS) {
			cd.geosets[9] = 1 + r.getUInt(ItemDisplayDB::GeosetB);
		}

		// boots
		if (type==IT_BOOTS) {
			cd.geosets[5] = 1 + r.getUInt(ItemDisplayDB::GeosetA);
		}

		// gloves
		if (type==IT_GLOVES) {
			cd.geosets[4] = 1 + r.getUInt(ItemDisplayDB::GeosetA);
		}

		// Tabard
		if (type == IT_TABARD) {
			cd.geosets[12] = 2;
		}

		// capes
		if (type==IT_CAPE) {
			cd.geosets[15] = 1 + r.getUInt(ItemDisplayDB::GeosetA);
			// load the cape texture
			const char *tex = r.getString(ItemDisplayDB::Skin);
			if (tex && strlen(tex)) capetex = texturemanager.add(AnimControl::makeSkinTexture("Item\\ObjectComponents\\Cape\\",tex));
		}

		// robe
		if (cd.geosets[13]==1) cd.geosets[13] = 1 + r.getUInt(ItemDisplayDB::GeosetC);
		if (cd.geosets[13]==2) {
			cd.geosets[5] = 0;		// hide the boots
			cd.geosets[9] = 0;		// hide the pants
			cd.geosets[12] = 0;		// also hide the tabard.
		}

		// gloves - this is so gloves have preference over shirt sleeves.
		// must be a more 'accurate' way of checking against this.
		if (cd.geosets[4] > 1) cd.geosets[8] = 0;

		tex.addLayer(makeItemTexture(CR_ARM_UPPER,r.getString(ItemDisplayDB::TexArmUpper)).c_str(),CR_ARM_UPPER, layer);
		tex.addLayer(makeItemTexture(CR_ARM_LOWER,r.getString(ItemDisplayDB::TexArmLower)).c_str(),CR_ARM_LOWER, layer);
		tex.addLayer(makeItemTexture(CR_TORSO_UPPER,r.getString(ItemDisplayDB::TexChestUpper)).c_str(),CR_TORSO_UPPER, layer);
		tex.addLayer(makeItemTexture(CR_TORSO_LOWER,r.getString(ItemDisplayDB::TexChestLower)).c_str(),CR_TORSO_LOWER, layer);
		tex.addLayer(makeItemTexture(CR_LEG_UPPER,r.getString(ItemDisplayDB::TexLegUpper)).c_str(),CR_LEG_UPPER, layer);
		tex.addLayer(makeItemTexture(CR_LEG_LOWER,r.getString(ItemDisplayDB::TexLegLower)).c_str(),CR_LEG_LOWER, layer);
		tex.addLayer(makeItemTexture(CR_HAND,r.getString(ItemDisplayDB::TexHands)).c_str(),CR_HAND, layer);
		tex.addLayer(makeItemTexture(CR_FOOT,r.getString(ItemDisplayDB::TexFeet)).c_str(),CR_FOOT, layer);

		// Display our customised tabard
		if (type==IT_TABARD && td.showCustom)	{
			tex.addLayer(td.GetBackgroundTex(CR_TORSO_UPPER).c_str(), CR_TORSO_UPPER, layer);
			tex.addLayer(td.GetBackgroundTex(CR_TORSO_LOWER).c_str(), CR_TORSO_LOWER, layer);
			tex.addLayer(td.GetIconTex(CR_TORSO_UPPER).c_str(), CR_TORSO_UPPER, layer);
			tex.addLayer(td.GetIconTex(CR_TORSO_LOWER).c_str(), CR_TORSO_LOWER, layer);
			tex.addLayer(td.GetBorderTex(CR_TORSO_UPPER).c_str(), CR_TORSO_UPPER, layer);
			tex.addLayer(td.GetBorderTex(CR_TORSO_LOWER).c_str(), CR_TORSO_LOWER, layer);
		}

	} catch (ItemDisplayDB::NotFound) {}
}

void CharControl::RefreshItem(int slot)
{
	// delete all attachments in that slot
	charAtt->delSlot(slot);

	int itemnum = cd.equipment[slot];
	if (itemnum!=0) {
		// load new model(s)
		int id1=-1, id2=-1;
		string path;
		//float sc = 1.0f;

		if (slot==CS_HEAD) {
			id1 = 11;
			path = "Item\\ObjectComponents\\Head\\";
		}
		else if (slot==CS_SHOULDER) {
			id1 = 6;
			id2 = 5;
			path = "Item\\ObjectComponents\\Shoulder\\";
		}
		else if (slot == CS_HAND_LEFT) 
			id1 = 2;
		else if (slot == CS_HAND_RIGHT) 
			id1 = 1;
		else 
			return;

		if (slot==CS_HAND_LEFT || slot==CS_HAND_RIGHT) {
			if (items.get(itemnum).type == IT_SHIELD) {
				path = "Item\\ObjectComponents\\Shield\\";
				id1 = 0;
			}
			else 
				path = "Item\\ObjectComponents\\Weapon\\";

			// If we're sheathing our weapons, relocate the items to
			// their correct positions
			if (bSheathe && items.get(itemnum).sheath>0)
			{	
				id1 = items.get(itemnum).sheath;
				if (id1==32 && slot==CS_HAND_LEFT)
					id1 = 33;
				
				if (id1==26 && items.get(itemnum).subclass==7 && slot==CS_HAND_RIGHT)
					id1 = 27;

				/*
				26 = upper right back
				27 = upper left back
				28 = center back
				30 = upside down, upper left back -- staves, spears
				32 = left hip
				33 = right hip
				*/
			}
		}

		try {
			const ItemRecord &item = items.get(itemnum);
			ItemDisplayDB::Record r = itemdb.getById(item.model);

			GLuint tex;
			string mp;
			bool succ = false;
			Attachment *att = NULL;
			Model *m = NULL;

			if (id1>=0) {
				mp = path + r.getString(ItemDisplayDB::Model);

				if (slot==CS_HEAD) {
					// sigh, head items have more crap to sort out
					mp = mp.substr(0, mp.length()-4); // delete .mdx
					mp.append("_");
					try {
						CharRacesDB::Record race = racedb.getById(cd.race);
						mp.append(race.getString(CharRacesDB::ShortName));
						mp.append(cd.gender?"F":"M");
						mp.append(".m2");
					} catch (CharRacesDB::NotFound) {
						mp = "";
					}
				}

				if (mp.length()) {
					att = charAtt->addChild(mp.c_str(), id1, slot);
					if (att) {
						m = static_cast<Model*>(att->model);
						if (m->ok) {
							mp = path + r.getString(ItemDisplayDB::Skin);
							mp.append(".blp");
							tex = texturemanager.add(mp);
							m->replaceTextures[2] = tex;
							succ = true;
						}
					}
				}
			}
			if (id2>=0) {
				mp = path + r.getString(ItemDisplayDB::Model2);
				if (mp.length()) {
					att = charAtt->addChild(mp.c_str(), id2, slot);
					if (att) {
						m = static_cast<Model*>(att->model);
						if (m->ok) {
							mp = path + r.getString(ItemDisplayDB::Skin2);
							mp.append(".blp");
							tex = texturemanager.add(mp);
							m->replaceTextures[2] = tex;
							succ = true;
						}
					}
				}
			}

			if (succ) {
				// okay, see if we have any glowy effects

				int visualid = r.getInt(ItemDisplayDB::Visuals);
				
				if (visualid == 0) {
					if ((modelViewer->enchants->RHandEnchant > -1) && (slot == CS_HAND_RIGHT)) {
						visualid = modelViewer->enchants->RHandEnchant;
					} else if ((modelViewer->enchants->LHandEnchant > -1) && (slot == CS_HAND_LEFT)) {
						visualid = modelViewer->enchants->LHandEnchant;
					}
				}

				if (m == NULL)
					m = static_cast<Model*>(att->model);

				if (visualid > 0) {
					try {
						ItemVisualDB::Record vis = visualdb.getById(visualid);
						for (int i=0; i<5; i++) {
							// try all five visual slots
							int effectid = vis.getInt(ItemVisualDB::Effect1 + i);
							if (effectid==0 || m->attLookup[i]<0) continue;

							try {
								ItemVisualEffectDB::Record eff = effectdb.getById(effectid);
								const char *filename = eff.getString(ItemVisualEffectDB::Model);

								att->addChild(filename, i, -1);

							} catch (ItemVisualEffectDB::NotFound) {}
						}
					} catch (ItemVisualDB::NotFound) {}
				}

			} else {
				cd.equipment[slot] = 0; // no such model? :(
			}

		} catch (ItemDisplayDB::NotFound) {}
	}
}

void CharControl::RefreshCreatureItem(int slot)
{
	// delete all attachments in that slot
	canvas->root->delSlot(slot);

	int itemnum = cd.equipment[slot];
	if (itemnum!=0) {
		// load new model(s)
		int id1=-1;
		string path;
		//float sc = 1.0f;

		if (slot == CS_HAND_LEFT) 
			id1 = 2;
		else if (slot == CS_HAND_RIGHT) 
			id1 = 1;
		else 
			return;

		if (slot==CS_HAND_LEFT || slot==CS_HAND_RIGHT) {
			if (items.get(itemnum).type == IT_SHIELD) {
				path = "Item\\ObjectComponents\\Shield\\";
				id1 = 0;
			} else {
				path = "Item\\ObjectComponents\\Weapon\\";
			}
		}

		try {
			const ItemRecord &item = items.get(itemnum);
			ItemDisplayDB::Record r = itemdb.getById(item.model);

			GLuint tex;
			string mp;
			bool succ = false;
			Attachment *att = NULL;
			Model *m = NULL;

			if (id1>=0) {
				mp = path + r.getString(ItemDisplayDB::Model);

				if (mp.length()) {
					att = canvas->root->addChild(mp.c_str(), id1, slot);
					if (att) {
						m = static_cast<Model*>(att->model);
						if (m->ok) {
							mp = path + r.getString(ItemDisplayDB::Skin);
							mp.append(".blp");
							tex = texturemanager.add(mp);
							m->replaceTextures[2] = tex;
							succ = true;
						}
					}
				}
			}

			if (succ) {
				// okay, see if we have any glowy effects
				int visualid = r.getInt(ItemDisplayDB::Visuals);
				
				if (visualid == 0) {
					if ((modelViewer->enchants->RHandEnchant > -1) && (slot == CS_HAND_RIGHT)) {
						visualid = modelViewer->enchants->RHandEnchant;
					} else if ((modelViewer->enchants->LHandEnchant > -1) && (slot == CS_HAND_LEFT)) {
						visualid = modelViewer->enchants->LHandEnchant;
					}
				}

				if (m == NULL)
					m = static_cast<Model*>(att->model);

				if (visualid > 0) {
					try {
						ItemVisualDB::Record vis = visualdb.getById(visualid);
						for (int i=0; i<5; i++) {
							// try all five visual slots
							int effectid = vis.getInt(ItemVisualDB::Effect1 + i);
							if (effectid==0 || m->attLookup[i]<0) continue;

							try {
								ItemVisualEffectDB::Record eff = effectdb.getById(effectid);
								const char *filename = eff.getString(ItemVisualEffectDB::Model);

								att->addChild(filename, i, -1);

							} catch (ItemVisualEffectDB::NotFound) {}
						}
					} catch (ItemVisualDB::NotFound) {}
				}

			} else {
				cd.equipment[slot] = 0; // no such model? :(
			}

		} catch (ItemDisplayDB::NotFound) {}
	}
}

std::string CharControl::makeItemTexture(int region, const char *name)
{
	// just return an empty filename
	if (!name || !strlen(name)) return "";

	// try gender-specific version first
	std::string fn = regionPaths[region];
	fn += name;
	fn += "_";
	fn += cd.gender ? "F" : "M";
	fn += ".blp";
	if (MPQFile::exists(fn.c_str())) return fn;

	// try unisex version
	fn[fn.length()-5] = 'U';
	return fn;
}

void CharTexture::compose(TextureID tex)
{
	// if we only have one texture then don't bother with compositing
	if (components.size()==1) {
		Texture temp(components[0].name);
		texturemanager.LoadBLP(tex, &temp);
		return;
	}

	std::sort(components.begin(), components.end());
	unsigned char destbuf[256*256*4], tempbuf[256*256*4];
	memset(destbuf, 0, 256*256*4);

	for (std::vector<CharTextureComponent>::iterator it = components.begin(); it != components.end(); ++it) {
		CharTextureComponent &comp = *it;
		const CharRegionCoords &coords = regions[comp.region];
		TextureID temptex = texturemanager.add(comp.name);
		Texture &tex = *((Texture*)texturemanager.items[temptex]);

		tex.getPixels(tempbuf);

		// blit the texture region over the original

		//assert(tex.w==coords.xsize && tex.h==coords.ysize);
		if (tex.w==coords.xsize && tex.h==coords.ysize) {
			for (int y=0, dy=coords.ypos; y<tex.h; y++,dy++) {
				for (int x=0, dx=coords.xpos; x<tex.w; x++,dx++) {
					unsigned char *src = tempbuf + y*tex.w*4 + x*4;
					unsigned char *dest = destbuf + dy*256*4 + dx*4;

					// this is slow and ugly but I don't care
					float r = src[3] / 255.0f;
					float ir = 1.0f - r;
					// zomg RGBA?
					dest[0] = (unsigned char)(dest[0]*ir + src[0]*r);
					dest[1] = (unsigned char)(dest[1]*ir + src[1]*r);
					dest[2] = (unsigned char)(dest[2]*ir + src[2]*r);
					dest[3] = 255;
				}
			}
		}

		texturemanager.del(temptex);
	}

	// good, upload this to video
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, destbuf);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
}

void CharDetails::reset()
{
	skinColor = 0;
	faceType = 0;
	hairColor = 0;
	hairStyle = 0;
	facialHair = 0;

	showUnderwear = true;
	showHair = true;
	showFacialHair = true;
	showEars = true;

	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		equipment[i] = 0;
	}
}


bool correctType(int type, int slot)
{
	if (type == IT_ALL) 
		return true;

	switch (slot) {
	case CS_HEAD:		return (type == IT_HEAD);
	case CS_NECK:		return (type == IT_NECK);
	case CS_SHOULDER:	return (type == IT_SHOULDER);
	case CS_SHIRT:		return (type == IT_SHIRT);
	case CS_CHEST:		return (type == IT_CHEST || type == IT_ROBE);
	case CS_BELT:		return (type == IT_BELT);
	case CS_PANTS:		return (type == IT_PANTS);
	case CS_BOOTS:		return (type == IT_BOOTS);
	case CS_BRACERS:	return (type == IT_BRACERS);
	case CS_GLOVES:		return (type == IT_GLOVES);
	//case CS_HAND_RIGHT:	return (type == IT_1HANDED || type == IT_GUN || type == IT_2HANDED || type == IT_CLAW || type == IT_DAGGER);
	//case CS_HAND_LEFT:	return (type == IT_1HANDED || type == IT_SHIELD || type == IT_DAGGER || type == IT_BOW);
	case CS_HAND_RIGHT:	return (type == IT_1HANDED || type == IT_GUN || type == IT_THROWN || type == IT_2HANDED || type == IT_CLAW || type == IT_DAGGER);
	case CS_HAND_LEFT:	return (type == IT_1HANDED || type == IT_BOW || type == IT_2HANDED || type == IT_SHIELD || type == IT_CLAW || type == IT_DAGGER || type == IT_OFFHAND);
	case CS_CAPE:		return (type == IT_CAPE);
	case CS_TABARD:		return (type == IT_TABARD);
	}
	return false;
}


void CharControl::ClearItemDialog()
{
	if (itemDialog) {
		itemDialog->Show(FALSE);
		delete itemDialog;
		itemDialog = 0;
	}
}

void CharControl::selectItem(int slot, int current, const wxChar *caption)
{
	ClearItemDialog();

	numbers.clear();
	choices.Clear();

	// collect all items for this slot, making note of the occurring subclasses
	set<pair<int,int> > subclassesFound;
	
	int sel=0, ord=0;
	for (std::vector<ItemRecord>::iterator it = items.items.begin(); it != items.items.end(); ++it) {
		if (correctType(it->type, slot)) {
			choices.Add(wxString(it->name, *wxConvCurrent));
			numbers.push_back(it->id);
			if (it->id == (unsigned int)current) sel = ord;
			ord++;

			if (it->itemclass > 0) 
				subclassesFound.insert(pair<int,int>(it->itemclass,it->subclass));
		}
	}

	// make category list
	cats.clear();
	catnames.clear();

	map<pair<int,int>, int> subclasslookup;
	for (ItemSubClassDB::Iterator it=subclassdb.begin(); it != subclassdb.end(); ++it) {
		int cl = it->getInt(ItemSubClassDB::ClassID);
		int scl = it->getInt(ItemSubClassDB::SubClassID);
		// only add the subclass if it was found in the itemlist
		if (cl>0 && subclassesFound.find(pair<int,int>(cl,scl)) != subclassesFound.end()) {
			
			// Used to go through the 'string fields' looking for the one with data.
			wxString str;
			for (int i = ItemSubClassDB::Name; i<18; i++)
			{
				str = wxString(it->getString(i), *wxConvCurrent);
				if (!str.IsEmpty()) {
					//p.name = str.fn_str();
					break;
				}
			}
			//string str = it->getString(ItemSubClassDB::Name);

			int hands = it->getInt(ItemSubClassDB::Hands);
			if (hands > 0) {
				char buf[16];
				sprintf(buf, " (%d-handed)", hands);
				str.append(buf);
			}
			catnames.Add(str.c_str());
			subclasslookup[pair<int,int>(cl,scl)] = (int)catnames.size()-1;
		}
	}

	if (subclassesFound.size() > 1) {
		// build category list
		for (size_t i=0; i<numbers.size(); i++) {
			ItemRecord r = items.get(numbers[i]);
			cats.push_back(subclasslookup[pair<int,int>(r.itemclass, r.subclass)]);
		}

		itemDialog = new CategoryChoiceDialog(this, UPDATE_ITEM, canvas->GetParent(), _T("Choose an item"), caption, choices, cats, catnames);
	} else {
		itemDialog = new FilteredChoiceDialog(this, UPDATE_ITEM, canvas->GetParent(), _T("Choose an item"), caption, choices);
	}

	itemDialog->SetSelection(sel);

	wxSize s = itemDialog->GetSize();
	const int w = 250;
	if (s.GetWidth() > w) {
		itemDialog->SetSizeHints(w,-1,-1,-1,-1,-1);
		itemDialog->SetSize(w, -1);
	}

	itemDialog->Move(itemDialog->GetParent()->GetPosition() + wxPoint(4,64));
	itemDialog->Show();
	choosingSlot = slot;
}

/*
struct NumStringPair {
	int id;
	string name;

	const bool operator< (const NumStringPair &p) const {
		return name < p.name;
	}
};
*/

void CharControl::selectSet()
{
	ClearItemDialog();

	std::vector<NumStringPair> items;

	for (ItemSetDB::Iterator it = sets.begin(); it != sets.end(); ++it) {
		int id = it->getUInt(ItemSetDB::SetID);
		if (sets.available(id)) {
			NumStringPair p;
			p.id = id;
			
			// Used to go through the 'string fields' looking for the one with data.
			for (int i = ItemSetDB::Name; i<9; i++)
			{
				//wxString str = wxString(it->getString(i), *wxConvCurrent);
				wxString str(wxCSConv("iso-8859-1").cWC2WX(wxConvUTF8.cMB2WC(it->getString(i))));
				if (!str.IsEmpty()) {
					p.name = str.fn_str();
					break;
				}
			}
			//p.name = it->getString(ItemSetDB::Name);

			items.push_back(p);
		}
	}

	std::sort(items.begin(), items.end());
	numbers.clear();
	choices.Clear();
	for (std::vector<NumStringPair>::iterator it = items.begin(); it != items.end(); ++it) {
		//choices.Add(wxString(it->name.c_str(), *wxConvCurrent));
		choices.Add(wxString(it->name.c_str(), *wxConvCurrent));
		numbers.push_back(it->id);
	}

	itemDialog = new FilteredChoiceDialog(this, UPDATE_SET, canvas->GetParent(), _("Choose an item set"), _("Item sets"), choices);
	itemDialog->Move(itemDialog->GetParent()->GetPosition() + wxPoint(4,64));
	itemDialog->Show();
}

void CharControl::selectStart()
{
	ClearItemDialog();

	numbers.clear();
	choices.Clear();

	for (StartOutfitDB::Iterator it = start.begin(); it != start.end(); ++it) {
		if ((it->getByte(StartOutfitDB::Race) == cd.race) && (it->getByte(StartOutfitDB::Gender) == cd.gender)) {
			try {
				CharClassesDB::Record r = classdb.getById(it->getByte(StartOutfitDB::Class));

				for (int i = CharClassesDB::Name; i<13; i++)
				{
					//wxString str = wxString(r.getString(i), *wxConvCurrent);
					wxString str(wxCSConv("iso-8859-1").cWC2WX(wxConvUTF8.cMB2WC(r.getString(i))));
					if (!str.IsEmpty()) {
						choices.Add(str);
						numbers.push_back(it->getUInt(StartOutfitDB::StartOutfitID));
						break;
					}
				}
			} catch (CharClassesDB::NotFound) {}
		}
	}

	itemDialog = new ChoiceDialog(this, UPDATE_START, canvas->GetParent(), _T("Choose a class"), _T("Classes"), choices);
	itemDialog->Move(itemDialog->GetParent()->GetPosition() + wxPoint(4,64));
	itemDialog->Show();
}

bool filterCreatures(string fn)
{
	wxString tmp(fn);
	tmp.LowerCase();
	const size_t len = tmp.Length();
	if (len < 4) return false;
	return (tmp[0]=='c' && tmp[1]=='r' && tmp[len-1]=='2' && tmp[len-2]=='m');
}

vector<wxString> creaturemodels;
vector<bool> ridablelist;

// TODO: Add an equivilant working version of this function for Linux / Mac OS X
void CharControl::selectMount()
{
#ifdef _WIN32
	ClearItemDialog();

	numbers.clear();
	choices.Clear();
	cats.clear();
	catnames.Clear();
	catnames.Add("Known ridable models");
	catnames.Add("Other models");

	static bool filelistInitialized = false;

	if (!filelistInitialized) {
		set<string> filelist;

		//#ifdef _WIN32
			hash_set<string> knownRidable;
		//#else
		//	__gnu_cxx::hash_set<std::string> knownRidable;
		//#endif

		getFileLists(filelist, filterCreatures);

		FILE *f = fopen("ridable.csv","r");
		if (f) {
			wxString tmp;
			while (!feof(f)) {
				char buf[1024];
				fgets(buf, 1024, f);
				size_t l = strlen(buf);
				if (buf[l-1]=='\n') 
					buf[l-1]=0;
				tmp = buf;
				tmp.LowerCase();
				knownRidable.insert(tmp.c_str());
				//creaturemodels.push_back(string(buf));
			}
		}
		
		for (set<string>::iterator it = filelist.begin(); it != filelist.end(); ++it) {
			wxString str(*it);
			creaturemodels.push_back(str);
			str.MakeLower();
			ridablelist.push_back(knownRidable.find(str)!=knownRidable.end());
		}
		filelistInitialized = true;
	}

	choices.Add("---- None ----");
	cats.push_back(0);
	
	for (size_t i=0; i<creaturemodels.size(); i++) {
		choices.Add(wxString(creaturemodels[i].c_str() + 9, *wxConvCurrent));
		cats.push_back(ridablelist[i]?0:1);
	}

	// TODO: obtain a list of "known ridable" models, and filter the list into two categories
	itemDialog = new FilteredChoiceDialog(this, UPDATE_MOUNT, canvas->GetParent(), _T("Choose a mount"), _T("Creatures"), choices);
	CategoryChoiceDialog *itemDialog = new CategoryChoiceDialog(this, UPDATE_MOUNT, canvas->GetParent(), _T("Choose a mount"), _T("Creatures"), choices, cats, catnames);
	itemDialog->Move(itemDialog->GetParent()->GetPosition() + wxPoint(4,64));
	itemDialog->Check(1, false);
	itemDialog->DoFilter();
	itemDialog->Show();

	// TODO: figure out why the damn dialog is so big and ugly
	const int w = 250;
	itemDialog->SetSizeHints(w,-1,-1,-1,-1,-1);
	itemDialog->SetSize(w, -1); 
	this->itemDialog = itemDialog;
#endif
}

void CharControl::OnUpdateItem(int type, int id)
{
	switch (type) {
	case UPDATE_ITEM:
		cd.equipment[choosingSlot] = numbers[id];
		if (slotHasModel(choosingSlot))
			RefreshItem(choosingSlot);
		labels[choosingSlot]->SetLabel(wxString(items.get(cd.equipment[choosingSlot]).name.c_str(), *wxConvCurrent));

		// Check if its a 'guild tabard'
		if (choosingSlot == CS_TABARD) 
			td.showCustom = (labels[choosingSlot]->GetLabel() == "Guild Tabard");

		break;
	case UPDATE_SET:
		id = numbers[id];
		if (id) {
			for (int i=0; i<NUM_CHAR_SLOTS; i++) {
				if (i!=CS_HAND_LEFT && i!=CS_HAND_RIGHT) cd.equipment[i] = 0;
			}
			cd.loadSet(sets, items, id);
			RefreshEquipment();
		}
		break;
	case UPDATE_START:
		id = numbers[id];
		if (id) {
			for (int i=0; i<NUM_CHAR_SLOTS; i++) cd.equipment[i] = 0;
			cd.loadStart(start, items, id);
			RefreshEquipment();
		}
		break;
	case UPDATE_MOUNT:
		if (canvas->root->model) {
			delete canvas->root->model;
			canvas->root->model = 0;
			canvas->model = 0;
		}

		if (id == 0) {
			// clearing the mount
			modelViewer->animControl->UpdateModel(model);
			model->isMounted = false;
			canvas->model = model;
			canvas->ResetView();
			charAtt->scale = canvas->root->scale;
			charAtt->id = 0;
		} else {
			//wxString fn(creaturemodels[id-1].c_str());
			Model *m = new Model(creaturemodels[id-1], true);

			// TODO: check if model is ridable
			canvas->root->model = m;
			canvas->model = m;
			modelViewer->animControl->UpdateModel(m);
			for (size_t i=0; i<model->header.nAnimations; i++) {
				if (model->anims[i].animID == 91) {
					model->animManager->Stop();
					model->currentAnim = (int)i;
					model->animManager->Set(0,(short)i,0);
					break;
				}
			}
			
			//model->currentAnim = 132;

			charAtt->parent = canvas->root;
			model->isMounted = true;
			//charAtt->id = 42;

			// Need to set this - but from what
			// Model data doesn't contain sizes for different race/gender
			// Character data doesn't contain sizes for different mounts
			// possibly some formula that from both models that needs to be calculated.
			// For "Taxi" mounts scale should be 1.0f I think, for now I'll ignore them
			// I really have no idea!  
			if(creaturemodels[id-1].substr(9,9) == "Kodobeast")
				charAtt->scale = 2.25f;
			else
				charAtt->scale = 1.0f;
			
			//Model *mChar = static_cast<Model*>(charAtt->model);
			//charAtt->scale = m->rad / mChar->rad;

			// Human Male = 2.0346599
			// NE Male = 2.5721216
			// NE Female = 2.2764397

			// RidingFrostSaber = 2.4360743
			// 1.00000

			//canvas->root->scale = 0.5f;
			//Attachment *att = charAtt->addChild("World\\ArtTest\\Boxtest\\xyz.m2", 24, -1);
			//m-> = att->scale;
			//delete att;

			canvas->ResetView();
			canvas->vRot.x = 0; // mounted characters look better from the side
		}
		break;

	case UPDATE_CREATURE_ITEM:
		cd.equipment[choosingSlot] = numbers[id];
		RefreshCreatureItem(choosingSlot);
		
		return;
	}

	if (canvas->modelType == MT_NPC)
		RefreshNPCModel();
	else
		RefreshModel();
}

void CharControl::OnTabardSpin(wxSpinEvent &event)
{
	if (canvas->modelType == MT_NPC)
		return;

	switch (event.GetId())
	{
	case ID_TABARD_ICON:
		td.Icon = event.GetPosition();
		break;
	case ID_TABARD_ICONCOLOR:
		td.IconColor = event.GetPosition();
		break;
	case ID_TABARD_BORDER:
        td.Border = event.GetPosition();
		if (td.Border > 5)
		{
			td.BorderColor = 0;
			tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetValue(0);
			tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetRange(0, 3);
		}
		else
			tabardSpins[SPIN_TABARD_BORDERCOLOR]->SetRange(0, 16);
		break;
	case ID_TABARD_BORDERCOLOR:
		td.BorderColor = event.GetPosition();
		break;
	case ID_TABARD_BACKGROUND:
		td.Background = event.GetPosition();
		break;
	}

	RefreshModel();
}

string TabardDetails::GetBackgroundTex(int slot)
{
	ostringstream tmpStream;

	string tmpU = "textures\\GuildEmblems\\Background_";
	if (Background < 10)
		tmpU += "0";
	tmpStream << Background;
	tmpU += tmpStream.str();
	tmpU += "_TU_U.blp";
	
	string tmpL = tmpU;
	tmpL[37] = 'L';

	if (slot == CR_TORSO_UPPER)
		return tmpU;
	else
		return tmpL;
}

string TabardDetails::GetBorderTex(int slot)
{
	ostringstream tmpStream;

	string tmpU = "textures\\GuildEmblems\\Border_";

	if (Border < 10)
		tmpU += "0";
	tmpStream << Border << "_";
	tmpU += tmpStream.str();
	tmpStream.flush();
	tmpStream.str("");

	if (BorderColor < 10)
		tmpU += "0";
	tmpStream << BorderColor;
	tmpU += tmpStream.str();

	tmpU += "_TU_U.blp";
	
	string tmpL = tmpU;
	tmpL[36] = 'L';

	if (slot == CR_TORSO_UPPER)
		return tmpU;
	else
		return tmpL;
}

string TabardDetails::GetIconTex(int slot)
{
	ostringstream tmpStream;

	string tmpU = "textures\\GuildEmblems\\Emblem_";

	if(Icon < 10)
		tmpU += "0";
	tmpStream << Icon << "_";
	tmpU += tmpStream.str();
	tmpStream.flush();
	tmpStream.str("");

	if(IconColor < 10)
		tmpU += "0";
	tmpStream << IconColor;
	tmpU += tmpStream.str();

	tmpU += "_TU_U.blp";
	
	string tmpL = tmpU;
	tmpL[tmpU.length() - 7] = 'L';

	if(slot == CR_TORSO_UPPER)
		return tmpU;
	else
		return tmpL;
}

void CharDetails::save(wxString fn, TabardDetails *td)
{
	// TODO: save/load as xml?
	// wx/xml/xml.h says the api will change, do not use etc etc.

	ofstream f(fn.fn_str(), ios_base::out|ios_base::trunc);
	f << race << " " << gender << endl;
	f << skinColor << " " << faceType << " " << hairColor << " " << hairStyle << " " << facialHair << " " << facialColor << endl;
	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		f << equipment[i] << endl;
	}

	// 5976 is the ID value for "Guild Tabard"
	if (equipment[CS_TABARD] == 5976) {
		f << td->Background << " " << td->Border << " " << td->BorderColor << " " << td->Icon << " " << td->IconColor << endl;
	}
	f.close();
}

bool CharDetails::load(wxString fn, TabardDetails *td)
{
	int r, g;
	ifstream f(fn.fn_str());
	f >> r >> g;
	bool same = false;
	if (r==race && g==gender) {
		f >> skinColor >> faceType >> hairColor >> hairStyle >> facialHair >> facialColor;
		same = true;
	} else {
		int dummy;
		for (int i=0; i<6; i++) f >> dummy;
	}
	for (int i=0; i<NUM_CHAR_SLOTS; i++) {
		f >> equipment[i];
	}

	// 5976 is the ID value for "Guild Tabard"
	if (equipment[CS_TABARD] == 5976 && !f.eof()) {
		f >> td->Background >> td->Border >> td->BorderColor >> td->Icon >> td->IconColor;
		td->showCustom = true;
	}

	f.close();
	return same;
}

void CharDetails::loadSet(ItemSetDB &sets, ItemDatabase &items, int setid)
{
	try {
		ItemSetDB::Record rec = sets.getById(setid);
		for (size_t i=0; i<ItemSetDB::NumItems; i++) {
			int id = rec.getInt(ItemSetDB::ItemIDBase + i);
			if (id==0) continue;
			const ItemRecord &r = items.get(id);
			if (r.type > 0) {
				// find a slot for it
				for (int s=0; s<NUM_CHAR_SLOTS; s++) {
					if (correctType(r.type, s)) {
						equipment[s] = id;
						break;
					}
				}
			}
		}
	} catch (ItemSetDB::NotFound) {}
}

void CharDetails::loadStart(StartOutfitDB &start, ItemDatabase &items, int setid)
{
	try {
		StartOutfitDB::Record rec = start.getById(setid);
		for (size_t i=0; i<StartOutfitDB::NumItems; i++) {
			int id = rec.getInt(StartOutfitDB::ItemIDBase + i);
			if (id==0) continue;
			const ItemRecord &r = items.get(id);
			if (r.type > 0) {
				// find a slot for it
				for (int s=0; s<NUM_CHAR_SLOTS; s++) {
					if (correctType(r.type, s)) {
						equipment[s] = id;
						break;
					}
				}
			}
		}
	} catch (ItemSetDB::NotFound) {}
}

const std::string CharControl::selectCharModel()
{
	wxArrayString arr;
	std::vector<int> ids;
	for (CharRacesDB::Iterator it = racedb.begin(); it != racedb.end(); ++it) {
		char buf[64];
		sprintf(buf,"%s Male", it->getString(CharRacesDB::FullName));
		arr.Add(buf);
		sprintf(buf,"%s Female", it->getString(CharRacesDB::FullName));
		arr.Add(buf);
		ids.push_back(it->getInt(CharRacesDB::RaceID));
	}
	wxSingleChoiceDialog dialog(this, _T("Choose a character model"), _T("Races"), arr);
	if (dialog.ShowModal() == wxID_OK) {
		int sel = dialog.GetSelection();
		int raceid = ids[sel >> 1];
		int gender = sel & 1;
		string genderStr = gender ? "Female" : "Male";
		try {
			CharRacesDB::Record r = racedb.getById(raceid);
			std::string path = "Character\\";
			path += r.getString(CharRacesDB::Name);
			path += "\\" + genderStr + "\\";
			path += r.getString(CharRacesDB::Name);
			path += genderStr + ".m2";
			return path;
		} catch (CharRacesDB::NotFound) {
			return ""; // wtf
		}
	}
	return "";
}

