#ifndef CHARDB_H
#define CHARDB_H
#include "dbcfile.h"

//#include <string>
#include "wx/string.h"

class CharHairGeosetsDB: public DBCFile
{
public:
	CharHairGeosetsDB(): DBCFile("DBFilesClient\\CharHairGeosets.dbc") {}
	~CharHairGeosetsDB() {}

	/// Fields
	static const size_t CharHairGeosetID = 0;	// uint
	static const size_t Race = 1;				// uint
	static const size_t Gender = 2;				// uint
	static const size_t Section = 3;			// uint
	static const size_t Geoset = 4;				// uint
	static const size_t Flags = 5;				// uint

	Record getByParams(unsigned int race, unsigned int gender, unsigned int section);
	int getGeosetsFor(unsigned int race, unsigned int gender);

private:

};

class CharSectionsDB: public DBCFile
{
public:
	CharSectionsDB(): DBCFile("DBFilesClient\\CharSections.dbc") {}
	~CharSectionsDB() {}

	/// Fields
	static const size_t SectonID = 0;	// uint
	static const size_t Race = 1;		// uint
	static const size_t Gender = 2;		// uint
	static const size_t Type = 3;		// uint
	static const size_t Section = 4;	// uint
	static const size_t Color = 5;		// uint
	static const size_t Tex1 = 6;		// string
	static const size_t Tex2 = 7;		// string
	static const size_t Tex3 = 8;		// string
	static const size_t IsNPC = 9;		// uint | 1 for npc

	/// Types
	static const size_t SkinType = 0;
	static const size_t FaceType = 1;
	static const size_t FacialHairType = 2;
	static const size_t HairType = 3;
	static const size_t UnderwearType = 4;

	Record getByParams(unsigned int race, unsigned int gender, unsigned int type, unsigned int section, unsigned int color, unsigned int npc);
	int getColorsFor(unsigned int race, unsigned int gender, unsigned int type, unsigned int section, unsigned int npc);
	int getSectionsFor(unsigned int race, unsigned int gender, unsigned int type, unsigned int color, unsigned int npc);
};


class CharRacesDB: public DBCFile
{
public:
	CharRacesDB(): DBCFile("DBFilesClient\\ChrRaces.dbc") {}
	~CharRacesDB() {}

	/// Fields
	static const size_t RaceID = 0;		// uint
	static const size_t ShortName = 6;	// string
	static const size_t Name = 15;		// string
	static const size_t FullName = 17;	// string
	static const size_t GeoType1 = 26;	// string
	static const size_t GeoType2 = 27;	// string
	static const size_t GeoType3 = 28;	// string

	Record getByName(wxString name);
	Record getById(unsigned int id);
};


class CharFacialHairDB: public DBCFile
{
public:
	CharFacialHairDB(): DBCFile("DBFilesClient\\CharacterFacialHairStyles.dbc") {}
	~CharFacialHairDB() {}

	/// Fields
	static const size_t Race = 0;				// uint
	static const size_t Gender = 1;				// uint
	static const size_t Style = 2;				// uint
	static const size_t Geoset100 = 6;			// uint
	static const size_t Geoset300 = 7;			// uint
	static const size_t Geoset200 = 8;			// uint
	
	Record getByParams(unsigned int race, unsigned int gender, unsigned int style);
	int getStylesFor(unsigned int race, unsigned int gender);

private:

};


class CharClassesDB: public DBCFile
{
public:
	CharClassesDB(): DBCFile("DBFilesClient\\ChrClasses.dbc") {}
	~CharClassesDB() {}

	/// Fields
	static const size_t ClassID = 0;	// uint
	static const size_t Name = 5;		// string - english name
	//static const size_t Name = 4;		// string - french name
	static const size_t RawName = 14;	// string

	Record getById(unsigned int id);
};


class HelmGeosetDB: public DBCFile
{
public:
	HelmGeosetDB(): DBCFile("DBFilesClient\\HelmetGeosetVisData.dbc") {}
	~HelmGeosetDB() {}

	/// Fields
	static const size_t TypeID = 0;		// uint
	static const size_t Field1 = 1;		// uint
	static const size_t Field2 = 2;		// uint
	static const size_t Field3 = 3;		// uint
	static const size_t Field4 = 4;		// uint
	static const size_t Field5 = 5;		// uint

	Record getById(unsigned int id);
};

#endif
